#ifndef _SQL_EDITOR_H_
#define _SQL_EDITOR_H_


#include <memory>
#include "wbpublic_public_interface.h"
#include "grtsqlparser/sql_syntax_check.h"
#include "grts/structs.db.mgmt.h"
#include "grt/grt_manager.h"
//!#include "interfaces/plugin.h"


#define Sql_VERSION "1.0"


class WBPUBLICBACKEND_PUBLIC_FUNC Sql_editor
{
public:
  static std::auto_ptr<Sql_editor> create_sql_editor(db_mgmt_RdbmsRef rdbms);

public:
  Sql_editor(db_mgmt_RdbmsRef rdbms);
  virtual ~Sql_editor();

  db_mgmt_RdbmsRef rdbms() { return _rdbms; }

  std::string sql() { return _sql; }
  void sql(const std::string &sql) { _sql= sql; }
  typedef Sql_parser_base::Parse_error_cb Sql_parser_err_cb;
  void sql_parser_err_cb(Sql_parser_err_cb cb);
  void check_syntax();

  int int_option(std::string name);
  std::string string_option(std::string name);

protected:
  db_mgmt_RdbmsRef _rdbms;
  grt::DictRef _options;
  std::string _sql;
  std::auto_ptr<Sql_syntax_check> _sql_syntax_check;
  Sql_parser_err_cb _sql_parser_err_cb;
};

#ifdef _WIN32
#pragma make_public(Sql_editor)
#endif

class WBPUBLICBACKEND_PUBLIC_FUNC Sql
{
public:
  virtual std::auto_ptr<Sql_editor> getSqlEditor(db_mgmt_RdbmsRef rdbms)= 0;
  virtual ~Sql() {}
};


#endif /* _SQL_EDITOR_H_ */
