
/*!
 Copyright 2009 Sun Microsystems, Inc.
 
 @author
 jak
 
 @class
 WBEditorTabView
 
 @abstract
 Implements the custom tabs used in editors. Small tabs at the bottom of the tab view, pointing downwards.
 
 @ingroup
 Custom Tab Views
 */



#import "WBEditorTabView.h"
#import "WBEditorTabItem.h"



@implementation WBEditorTabView



- (void) drawRect: (NSRect) frame;
{
  [super drawRect: frame];
  
  NSRect r = [self bounds];
  r.origin.y += [self tabAreaHeight];
  
  NSColor* c = [NSColor colorWithCalibratedRed: 0.88
                                         green: 0.88
                                          blue: 0.88
                                         alpha: 1];
  [c set];
  [NSBezierPath fillRect: r];
  
  c = [NSColor colorWithCalibratedRed: 0.3
                                green: 0.3
                                 blue: 0.3
                                alpha: 1];
  [c set];
  [NSBezierPath strokeLineFromPoint: r.origin
                            toPoint: NSMakePoint(r.origin.x, r.size.height)];
  [NSBezierPath strokeLineFromPoint: NSMakePoint(r.origin.x, r.size.height)
                            toPoint: NSMakePoint(r.size.width, r.size.height)];
  [NSBezierPath strokeLineFromPoint: NSMakePoint(r.size.width, r.size.height)
                            toPoint: NSMakePoint(r.size.width, r.origin.y)];
  [NSBezierPath strokeRect: r];
}



- (float) contentPadding;
{
  return 3;
}



- (CGColorRef) tabRowActiveBackgroundColorCreate;
{
	return nil;
}



- (CGColorRef) tabRowInactiveBackgroundColorCreate;
{
	return nil;
}



- (CALayer*) shadowLayer;
{
  CALayer* shadowLayer = [super shadowLayer];
  [shadowLayer setShadowOpacity: 0.3];
	[shadowLayer setShadowOffset: CGSizeMake(0, -0.5)];
  
  return shadowLayer;
}



- (WBTabItem*) tabItemWithIdentifier: (id) identifier
                               label: (NSString*) label;
{
	WBTabItem* item = [WBEditorTabItem tabItemWithIdentifier: identifier
                                                        label: label];
	
	return item;
}


- (void) doCustomize;
{
	mTabPlacement = WBTabPlacementBottom;
	mTabDirection = WBTabDirectionDown;
	mTabSize = WBTabSizeSmall;
	
	[super doCustomize];
}



@end


