//
//  WBOverviewListController.mm
//  MySQLWorkbench
//
//  Created by Alfredo Kojima on 12/Oct/08.
//  Copyright 2008 Sun Microsystems Inc. All rights reserved.
//

#import "WBOverviewListController.h"
#import "GRTIconCache.h"

#import "MCollectionViewItemView.h"

#include "workbench/wb_context.h" // temporary

@implementation WBOverviewListController

- (id)init
{
  if ((self= [super init]) != nil)
  {
    _selectedIndexes= nil;
  }
  return self;
}


- (void)dealloc
{
  [[NSNotificationCenter defaultCenter] removeObserver:self];
  
  [_selectedIndexes release];
  [_items release];
  
  [super dealloc];
}


- (void)setCollectionView:(NSCollectionView*)view
{
  collectionView= view;
  
  [[NSNotificationCenter defaultCenter] addObserver:self
                                           selector:@selector(collectionFrameChanged:)
                                               name:NSViewFrameDidChangeNotification
                                             object:view];
}


- (void)relayoutCollectionView:(NSCollectionView*)collection
{
  int count= [_items count];
  NSRect itemRect= [[[collection itemPrototype] view] frame];
  int itemsPerRow;
  int rowCount= 0;
  NSSize newSize;

  itemsPerRow= (int)floor(NSWidth([collection frame]) / NSWidth(itemRect));
  if (itemsPerRow > 0)
  {
    rowCount= ceil((count + (itemsPerRow/2)) / itemsPerRow);
    if (rowCount * itemsPerRow < count)
      rowCount++;
  }
  if (rowCount == 0)
    rowCount= 1;

  newSize.width= NSWidth([collection frame]);
  newSize.height= NSHeight(itemRect) * rowCount;

  if (!NSEqualSizes([collection frame].size, newSize))
    [collection setFrameSize:newSize];
  
  [collection setNeedsDisplay:YES];
}


- (void)collectionFrameChanged:(NSNotification*)notif
{
  NSCollectionView *collection= [notif object];
  
  [self relayoutCollectionView:collection];
}


- (void)fillFromChildrenOf:(const bec::NodeId&)node
                  overview:(wb::OverviewBE*)overview
                  iconSize:(bec::IconSize)iconSize
{
  NSMutableArray *items= [NSMutableArray array];
  GRTIconCache *iconCache= [GRTIconCache sharedIconCache];
    
  for (size_t c= overview->count_children(node), i= 0; i < c; i++)
  {
    bec::NodeId child= overview->get_child(node, i);
    NSImage *icon;
    std::string name;

    overview->get_field(child, wb::OverviewBE::Label, name);
    icon= [iconCache imageForIconId:overview->get_field_icon(child, wb::OverviewBE::Label, iconSize)];
    if (!icon)
      icon= [NSImage imageNamed:@"MySQLWorkbench-16.png"];

    [items addObject:[NSDictionary dictionaryWithObjectsAndKeys:
                      [NSString stringWithUTF8String:name.c_str()], @"name",
                      [NSString stringWithCString:child.repr().c_str()], @"path",
                      icon, @"image", // put image as last because it can be nil if the image doesnt exist
                      nil]];
  }

  [self setItems:items];

  [self relayoutCollectionView:collectionView];
}

- (void)setShowLargeIcons:(BOOL)flag
{
  if (flag)
    [collectionView setItemPrototype:largeIcon];
  else
    [collectionView setItemPrototype:smallIcon];
}


- (void)setItems:(NSMutableArray*)items
{
  if (_items != items)
  {
    [_items release];
    _items= [items retain];
  }
}


- (NSIndexSet*)selectedIndexes
{
  return _selectedIndexes;
}


- (void)setSelectedIndexes:(NSIndexSet*)indexes
{  
  if (_selectedIndexes != indexes)
  {    
    [_selectedIndexes release];
    _selectedIndexes= [indexes retain];

    if (_selectedIndexes)
    {
      _overview->begin_selection_marking();
      if ([_selectedIndexes count] > 0)
      {
        for (NSUInteger i= [_selectedIndexes firstIndex]; i <= [_selectedIndexes lastIndex]; i= [_selectedIndexes indexGreaterThanIndex:i])
        {
          NSDictionary *item= [_items objectAtIndex:i];
          bec::NodeId node;
          NSString *path= [item objectForKey:@"path"];
          node= bec::NodeId([path UTF8String]);
          _overview->select_node(node);
        }
      }
      _overview->end_selection_marking();
    }    
  }
}


- (void)activateCollectionItem:(id)sender
{
  if ([_selectedIndexes count] == 1)
  {
    NSDictionary *item= [_items objectAtIndex:[_selectedIndexes firstIndex]];
    bec::NodeId node;
    NSString *path= [item objectForKey:@"path"];
        
    node= bec::NodeId([path UTF8String]);
    
    _overview->activate_node(node);
  }
}


- (void)selectCollectionItem:(id)sender
{
}




- (void)setOverviewBE:(wb::OverviewBE*)overview
{
  _overview= overview;
}


- (NSMutableArray*)items
{
  return _items;
}


@end
