#ifndef _LF_BUTTON_H_
#define _LF_BUTTON_H_


#include "mforms/button.h"

#include "lf_view.h"


namespace mforms {
namespace gtk {

class ButtonImpl : public ViewImpl
{
 protected:
  Gtk::Button *_button;
  virtual Gtk::Widget *get_outer() const { return _button; }

  ButtonImpl(::mforms::Button *self)
    : ViewImpl(self)
  {
    _button= Gtk::manage(new Gtk::Button("", true));
    _button->signal_clicked().connect(sigc::mem_fun(*self, &::mforms::Button::callback));
    
    setup();
  }

  static bool create(::mforms::Button *self)
  {
    return new ButtonImpl(self) != 0;
  }

  static void set_text(::mforms::Button *self, const std::string &text)
  {
    if ( self )
    {
      ButtonImpl* button = self->get_data<ButtonImpl>();

      if ( button )
      {
        button->_button->set_label(text);
      }
    }
  }

  static void enable_internal_padding(Button *self, bool enabled)
  {
    ButtonImpl* button = self->get_data<ButtonImpl>();
    if (button && enabled)
    {
      Gtk::Alignment *align= Gtk::manage(new Gtk::Alignment());
      Gtk::Widget *content= button->_button->get_child();
      content->reference();
      button->_button->remove();
      align->add(*content);
      content->unreference();
      button->_button->add(*align);

      align->property_left_padding()= 6;
      align->property_right_padding()= 6;
      
      align->show_all();
    }
  }

public:
  static void init()
  {
    ::mforms::ControlFactory *f = ::mforms::ControlFactory::get_instance();

    f->_button_impl.create= &ButtonImpl::create;
    f->_button_impl.set_text= &ButtonImpl::set_text;
    f->_button_impl.enable_internal_padding= &ButtonImpl::enable_internal_padding;
  }
};

}
}

#endif
