#ifndef _MFORMS_BASE_H_
#define _MFORMS_BASE_H_

#include <string>
#include <list>

#ifdef _WIN32
# ifdef DECL_MFORMS_EXPORT
#  define MFORMS_EXPORT __declspec(dllexport)
# else
#  define MFORMS_EXPORT __declspec(dllimport)
# endif
#else
# define MFORMS_EXPORT
#endif

#ifndef _WIN32
#define __stdcall 
#endif

#ifdef __APPLE__
#ifdef nil
#undef nil
#endif 
#include <sigc++/sigc++.h>
#define nil __DARWIN_NULL
#include <objc/objc-runtime.h>
#endif

namespace mforms {
  class ControlFactory;
  
  //!
  //!
  //!
  class MFORMS_EXPORT Object
  {
    
    //! Note: set_data and get_data should be used exclusivelly by the implementation code
    //! for each platform.
#ifdef __APPLE__
  public:
    Object() : _data(nil) {}
    void set_data(id data) { objc_msgSend(data, sel_getUid("retain")); /* calls [data retain] */ _data= data; }
    id get_data() { return _data; }
    
    virtual ~Object() { /*objc_msgSend(_data, sel_getUid("release"));*/ /* calls [_data release] */ }
  private:
    id _data;
#else
  public:
    virtual ~Object() { }

    void set_data(void *data) { _data= data; }

    template<class C>
      C* get_data() { return reinterpret_cast<C*>(_data); }

    void *get_data_ptr() { return _data; }

  private:
    void *_data;
#endif
  };
};

#endif
