
// Copyright (c) 2009, Sun Microsystems, Inc.

#include "stdafx.h"
#include "wf_textentry.h"

using namespace MySQL::Forms;

//--------------------------------------------------------------------------------------------------

bool TextEntryImpl::create(::mforms::TextEntry *self, bool password)
{
  TextEntryImpl ^entry= gcnew TextEntryImpl(self);

  if (entry != nullptr)
  {
    TextBox ^tb= ViewImpl::create<TextBox>(self, entry);
    tb->TextChanged += gcnew EventHandler(change);
    tb->Multiline= false;
    if (password)
      tb->PasswordChar= '*';
    tb->Size= Size(100, tb->PreferredSize.Height); // DefaultSize is not accessible here. 
    return true;
  }
  return false;
}

//--------------------------------------------------------------------------------------------------

void TextEntryImpl::set_text(::mforms::TextEntry *self, const std::string &text)
{
  TextEntryImpl^ cb= (TextEntryImpl^)ObjectImpl::FromUnmanaged(self);

  if (cb != nullptr)
  {
    TextBox^ box= cb->get_control<TextBox>();
    box->Text= CppStringToNative(text);
   // textentry shouldn't size to fit box->Size= box->PreferredSize;
  }
}

//--------------------------------------------------------------------------------------------------

std::string TextEntryImpl::get_text(::mforms::TextEntry *self)
{
  TextEntryImpl^ cb= (TextEntryImpl^)ObjectImpl::FromUnmanaged(self);

  if (cb != nullptr)
  {
    return NativeToCppString(cb->get_control<TextBox>()->Text);
  }
  return "";
}

//--------------------------------------------------------------------------------------------------

void TextEntryImpl::set_password(::mforms::TextEntry *self, bool flag)
{
  TextEntryImpl^ cb= (TextEntryImpl^)ObjectImpl::FromUnmanaged(self);

  if (cb != nullptr)
  {
    if (flag)
      cb->get_control<TextBox>()->PasswordChar= '*';
    else
      cb->get_control<TextBox>()->PasswordChar= 0;
  }
}

//--------------------------------------------------------------------------------------------------

void TextEntryImpl::set_max_length(::mforms::TextEntry *self, int length)
{
  TextEntryImpl^ cb= (TextEntryImpl^)ObjectImpl::FromUnmanaged(self);

  if (cb != nullptr)
  {
    cb->get_control<TextBox>()->MaxLength= length;
  }
}

//--------------------------------------------------------------------------------------------------

void TextEntryImpl::change(Object^ sender, EventArgs^ args)
{
  TextBox^ tb= (TextBox^) sender;

  if (tb->Tag != nullptr)
  {
    ::mforms::TextEntry* box= ViewImpl::get_backend_control<::mforms::TextEntry>(tb);
    if (box != NULL)
      box->callback();
  }
}

//--------------------------------------------------------------------------------------------------

TextEntryImpl::TextEntryImpl(::mforms::TextEntry *text)
  : ViewImpl(text)
{
}

//--------------------------------------------------------------------------------------------------
