#include "wb_config.h"
#include "linux_utilities/plugin_editor_base.h"
#include "../backend/mysql_view_editor.h"
#include "linux_utilities/MGGladeXML.h"
#include "grtdb/db_object_helpers.h"
#include "treemodel_wrapper.h"
#include "sql_editor_fe.h"
#include "errors_list.h"

//==============================================================================
//
//==============================================================================
class DbMySQLViewEditor : public PluginEditorBase
{
  MySQLViewEditorBE                 _be;
  
  virtual bec::BaseEditor *get_be();
  
 public:
  DbMySQLViewEditor(grt::Module *m, bec::GRTManager *grtm, const grt::BaseListRef &args);
  
  virtual ~DbMySQLViewEditor();
  virtual void do_refresh_form_data();
  virtual std::string get_title();

 private:
  void sql_changed(const std::string& sql);
   
  SqlEditorFE           _sql_editor;
  ErrorsList            _errors;
  TrackedPane          *_pane;
};

DbMySQLViewEditor::DbMySQLViewEditor(grt::Module *m, bec::GRTManager *grtm, const grt::BaseListRef &args)
    : PluginEditorBase(m ,grtm, args, "modules/data/editor_view.glade")
    , _be(grtm, db_mysql_ViewRef::cast_from(args[0]), get_rdbms_for_db_object(args[0]))
    , _errors(&_be)
{
  Gtk::Notebook *editor_window(0);
  xml()->get("mysql_view_editor_notebook", &editor_window);

  Gtk::Image *image;
  xml()->get("view_editor_image", &image);
  image->set(ImageCache::get_instance()->image_from_filename("db.View.editor.48x48.png", false));
  xml()->get("view_editor_image2", &image);
  image->set(ImageCache::get_instance()->image_from_filename("db.View.editor.48x48.png", false));

  _be.set_refresh_ui_slot(sigc::mem_fun(this, &DbMySQLViewEditor::refresh_form_data));
  
  editor_window->reparent(*this);
  editor_window->show();

  Gtk::TextView *tview(0);
  xml()->get("viewcomment", &tview);
  add_text_change_timer(tview, sigc::mem_fun(_be, &MySQLViewEditorBE::set_comment));

  Gtk::Frame *ddl_win;
  xml()->get("view_ddl", &ddl_win);
  _sql_editor.widget().set_size_request(-1, 100);
  ddl_win->add(_sql_editor.widget());
  add_sqleditor_text_change_timer(&_sql_editor, sigc::mem_fun(this, &DbMySQLViewEditor::sql_changed));
  _errors.set_sql_parser_err_cb(sigc::mem_fun(&_sql_editor, &SqlEditorFE::process_syntax_error));
  ddl_win->resize_children();

  Gtk::ScrolledWindow* win;
  xml()->get("errors_holder", &win);
  win->add(_errors.widget());
  //_errors.signal_error_selected().connect(sigc::mem_fun(this, &DbMySQLTableEditorTriggerPage::error_selected));

  Gtk::Paned *paned(0);
  xml()->get("view_paned", &paned);
  _pane = new TrackedPane(paned, &_be);

  
  refresh_form_data();
  _pane->max1st();

  std::string query(_be.get_query());
  _sql_editor.set_text(query);

  show_all();
}

//------------------------------------------------------------------------------
DbMySQLViewEditor::~DbMySQLViewEditor()
{
  delete _pane;
}

//------------------------------------------------------------------------------
std::string DbMySQLViewEditor::get_title()
{
  return strfmt(_("View: %s"), _be.get_name().c_str());
}

//------------------------------------------------------------------------------
bec::BaseEditor *DbMySQLViewEditor::get_be()
{
  return &_be;
}

//------------------------------------------------------------------------------
void DbMySQLViewEditor::do_refresh_form_data()
{
  Gtk::Entry* entry(0);
  xml()->get("view_name", &entry);
  if (entry->get_text() != _be.get_name())
  {
    entry->set_text(_be.get_name());
    _signal_title_changed.emit(get_title());
  }

  Gtk::TextView *tview(0);
  xml()->get("viewcomment", &tview);
  tview->get_buffer()->set_text(_be.get_comment());
  
}

//------------------------------------------------------------------------------
void DbMySQLViewEditor::sql_changed(const std::string& sql)
{
  _pane->max1st();
  _errors.clear_list();
  _sql_editor.reset_syntax_errors();
  _be.set_query(sql);
}

//------------------------------------------------------------------------------
extern "C" 
{
  GUIPluginBase *createDbMysqlViewEditor(grt::Module *m, bec::GRTManager *grtm, const grt::BaseListRef &args)
  {
    return Gtk::manage(new DbMySQLViewEditor(m, grtm, args));
  }
};
