//
//  WBModelStoredNoteEditor.mm
//  MySQLWorkbench
//
//  Created by Alfredo Kojima on 19/Feb/09.
//  Copyright 2009 Sun Microsystems Inc. All rights reserved.
//

#import "WbModelStoredNoteEditor.h"

#import "MCPPUtilities.h"

#import "CCSciView.h"

@implementation StoredNoteEditor

static void call_refresh(StoredNoteEditor *self)
{
  [self refresh];
}


- (id)initWithModule:(grt::Module*)module GRTManager:(bec::GRTManager*)grtm arguments:(const grt::BaseListRef&)args
{
  self= [super initWithModule:module GRTManager:grtm arguments:args];
  if (self != nil)
  {
    
    
    // load GUI. Top level view in the nib is the NSTabView that will be docked to the main window
    if (![[NSBundle bundleForClass:[self class]] loadNibFile:@"WbModelStoredNoteEditor"
                                           externalNameTable:[NSDictionary dictionaryWithObject:self forKey:NSNibOwner] 
                                                    withZone:nil])
      NSLog(@"Could not load WbModelStoredNoteEditor.xib");
    
    // take the minimum size of the view from the initial size in the nib.
    // Therefore the nib should be designed as small as possible
    // note: the honouring of the min size is not yet implemented
    [self setMinimumSize: [tabView frame].size];
    
    // setup the editor backend with the note object (args[0])
    mBackEnd= new StoredNoteEditorBE(grtm, GrtStoredNoteRef::cast_from(args[0]));
    
    // register a callback that will make [self refresh] get called
    // whenever the backend thinks its needed to refresh the UI from the backend data (ie, the
    // edited object was changed from somewhere else in the application)
    mBackEnd->set_refresh_ui_slot(sigc::bind(sigc::ptr_fun(call_refresh), self));
        
    // update the UI
    [self refresh];
  }
  return self;
}


- (void) dealloc
{
  delete mBackEnd;
  [super dealloc];
}


/** Fetches object info from the backend and update the UI
 */
- (void)refresh
{
  if (mBackEnd)
  {
    bool isutf8= true;
    std::string data= mBackEnd->get_text(isutf8);
    [nameText setStringValue: [NSString stringWithCPPString:mBackEnd->get_name()]];
    
    if (isutf8)
    {
      [noteText setString: [NSString stringWithCPPString:data]];
      [noteText setEditable: YES];
    }
    else
    {
      [noteText setEditable: NO];
      [noteText setString: NSLocalizedString(@"Data is not UTF8 and cannot be displayed.", @"attached file editor")];
    }
    [applyButton setEnabled: NO];
    [revertButton setEnabled: NO];
  }
}


- (id)identifier
{
  // an identifier for this editor (just take the object id)
  return [NSString stringWithCPPString:mBackEnd->get_object().id()];
}


- (NSString*)title
{
  // the title for the editor
  return [NSString stringWithFormat:NSLocalizedString(@"%s - Attached File", @"editor tab title"), mBackEnd->get_name().c_str()];
}


- (NSView*)dockableView
{
  // the view to be docked to the main window
  return tabView;
}


- (BOOL)matchesIdentifierForClosingEditor:(NSString*)identifier
{
  return mBackEnd->should_close_on_delete_of([identifier UTF8String]);
}


- (void)controlTextDidEndEditing:(NSNotification *)aNotification
{
  if ([aNotification object] == nameText)
  {
    // set name of the note
    mBackEnd->set_name([[nameText stringValue] UTF8String]);
  }  
}


- (IBAction)loadText:(id)sender
{
  NSOpenPanel *panel= [NSOpenPanel openPanel];
  
  [panel setTitle: NSLocalizedString(@"Load from File", @"loadText from file dialog")];
  if ([panel runModal] == NSOKButton)
  {
    mBackEnd->load_file([[panel filename] UTF8String]);
    [self refresh];
  }
}


- (IBAction)saveText:(id)sender
{
  NSSavePanel *panel= [NSSavePanel savePanel];
  
  [panel setTitle: NSLocalizedString(@"Save to File", @"saveText to file dialog")];
  if ([panel runModal] == NSOKButton)
  {
    mBackEnd->save_file([[panel filename] UTF8String]);
  }
}


- (IBAction)applyChanges:(id)sender
{
  mBackEnd->set_text([[noteText string] UTF8String]);
}


- (IBAction)revertChanges:(id)sender
{
  [self refresh];
}



- (void)textDidChange:(NSNotification *)aNotification
{
  if ([aNotification object] == noteText)
  {
    [applyButton setEnabled: YES];
    [revertButton setEnabled: YES];
  }
}

@end
